=== modified file 'NEWS'
--- NEWS	2009-03-10 00:49:23 +0000
+++ NEWS	2009-03-10 01:29:23 +0000
@@ -38,6 +38,8 @@
     * Multiple authors for a commit can now be recorded by using the "--author"
       option multiple times. (James Westby, #185772)
 
+    * New clean-tree command, from bzrtools.  (Aaron Bentley, Jelmer Vernoij)
+
     * New command ``bzr launchpad-open`` opens a Launchpad web page for that
       branch in your web browser, as long as the branch is on Launchpad at all.
       (Jonathan Lange)

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2009-03-08 23:40:12 +0000
+++ bzrlib/builtins.py	2009-03-10 01:29:23 +0000
@@ -5455,6 +5455,40 @@
         Unshelver.from_args(shelf_id, action).run()
 
 
+class cmd_clean_tree(Command):
+    """Remove unwanted files from working tree.
+
+    By default, only unknown files, not ignored files, are deleted.  Versioned
+    files are never deleted.
+
+    Another class is 'detritus', which includes files emitted by bzr during
+    normal operations and selftests.  (The value of these files decreases with
+    time.)
+
+    If no options are specified, unknown files are deleted.  Otherwise, option
+    flags are respected, and may be combined.
+
+    To check what clean-tree will do, use --dry-run.
+    """
+    takes_options = [Option('ignored', help='Delete all ignored files.'),
+                     Option('detritus', help='Delete conflict files, merge'
+                            ' backups, and failed selftest dirs.'),
+                     Option('unknown',
+                            help='Delete files unknown to bzr (default).'),
+                     Option('dry-run', help='Show files to delete instead of'
+                            ' deleting them.'),
+                     Option('force', help='Do not prompt before deleting.')]
+    def run(self, unknown=False, ignored=False, detritus=False, dry_run=False,
+            force=False):
+        from bzrlib.clean_tree import clean_tree
+        if not (unknown or ignored or detritus):
+            unknown = True
+        if dry_run:
+            force = True
+        clean_tree('.', unknown=unknown, ignored=ignored, detritus=detritus,
+                   dry_run=dry_run, no_prompt=force)
+
+
 def _create_prefix(cur_transport):
     needed = [cur_transport]
     # Recurse upwards until we can create a directory successfully

=== added file 'bzrlib/clean_tree.py'
--- bzrlib/clean_tree.py	1970-01-01 00:00:00 +0000
+++ bzrlib/clean_tree.py	2009-03-09 04:36:37 +0000
@@ -0,0 +1,85 @@
+# Copyright (C) 2005 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import errno
+import os
+import shutil
+import sys
+
+from bzrlib.osutils import has_symlinks, isdir
+from bzrlib.trace import note
+from bzrlib.workingtree import WorkingTree
+
+
+def is_detritus(subp):
+    """Return True if the supplied path is detritus, False otherwise"""
+    return subp.endswith('.THIS') or subp.endswith('.BASE') or\
+        subp.endswith('.OTHER') or subp.endswith('~') or subp.endswith('.tmp')
+
+
+def iter_deletables(tree, unknown=False, ignored=False, detritus=False):
+    """Iterate through files that may be deleted"""
+    for subp in tree.extras():
+        if detritus and is_detritus(subp):
+            yield tree.abspath(subp), subp
+            continue
+        if tree.is_ignored(subp):
+            if ignored:
+                yield tree.abspath(subp), subp
+        else:
+            if unknown:
+                yield tree.abspath(subp), subp
+
+
+def clean_tree(directory, unknown=False, ignored=False, detritus=False,
+               dry_run=False, no_prompt=False):
+    """Remove files in the specified classes from the tree"""
+    tree = WorkingTree.open_containing(directory)[0]
+    tree.lock_read()
+    try:
+        deletables = list(iter_deletables(tree, unknown=unknown,
+            ignored=ignored, detritus=detritus))
+        if len(deletables) == 0:
+            note('Nothing to delete.')
+            return 0
+        if not no_prompt:
+            for path, subp in deletables:
+                print subp
+            val = raw_input('Are you sure you wish to delete these [y/N]?')
+            if val.lower() not in ('y', 'yes'):
+                print 'Canceled'
+                return 0
+        delete_items(deletables, dry_run=dry_run)
+    finally:
+        tree.unlock()
+
+
+def delete_items(deletables, dry_run=False):
+    """Delete files in the deletables iterable"""
+    has_deleted = False
+    for path, subp in deletables:
+        if not has_deleted:
+            note("deleting paths:")
+            has_deleted = True
+        note('  ' + subp)
+        if not dry_run:
+            if isdir(path):
+                shutil.rmtree(path)
+            else:
+                os.unlink(path)
+    if not has_deleted:
+        note("No files deleted.")

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2009-03-09 08:45:56 +0000
+++ bzrlib/tests/__init__.py	2009-03-10 01:29:23 +0000
@@ -2918,6 +2918,7 @@
                    'bzrlib.tests.test_bundle',
                    'bzrlib.tests.test_bzrdir',
                    'bzrlib.tests.test_cache_utf8',
+                   'bzrlib.tests.test_clean_tree',
                    'bzrlib.tests.test_chunk_writer',
                    'bzrlib.tests.test__chunks_to_lines',
                    'bzrlib.tests.test_commands',

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2009-03-07 06:58:17 +0000
+++ bzrlib/tests/blackbox/__init__.py	2009-03-10 01:29:23 +0000
@@ -47,6 +47,7 @@
                      'bzrlib.tests.blackbox.test_cat_revision',
                      'bzrlib.tests.blackbox.test_check',
                      'bzrlib.tests.blackbox.test_checkout',
+                     'bzrlib.tests.blackbox.test_clean_tree',
                      'bzrlib.tests.blackbox.test_command_encoding',
                      'bzrlib.tests.blackbox.test_commit',
                      'bzrlib.tests.blackbox.test_conflicts',

=== added file 'bzrlib/tests/blackbox/test_clean_tree.py'
--- bzrlib/tests/blackbox/test_clean_tree.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_clean_tree.py	2009-03-10 00:33:54 +0000
@@ -0,0 +1,64 @@
+# Copyright (C) 2005, 2009 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+
+
+"""Tests of the 'bzr clean-tree' command."""
+
+
+import os
+
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestBzrTools(TestCaseWithTransport):
+
+    @staticmethod
+    def touch(filename):
+        my_file = open(filename, 'wb')
+        try:
+            my_file.write('')
+        finally:
+            my_file.close()
+
+    def test_clean_tree(self):
+        self.run_bzr('init')
+        self.run_bzr('ignore *~')
+        self.run_bzr('ignore *.pyc')
+        self.touch('name')
+        self.touch('name~')
+        self.failUnlessExists('name~')
+        self.touch('name.pyc')
+        self.run_bzr('clean-tree --force')
+        self.failUnlessExists('name~')
+        self.failIfExists('name')
+        self.touch('name')
+        self.run_bzr('clean-tree --detritus --force')
+        self.failUnlessExists('name')
+        self.failIfExists('name~')
+        self.failUnlessExists('name.pyc')
+        self.run_bzr('clean-tree --ignored --force')
+        self.failUnlessExists('name')
+        self.failIfExists('name.pyc')
+        self.run_bzr('clean-tree --unknown --force')
+        self.failIfExists('name')
+        self.touch('name')
+        self.touch('name~')
+        self.touch('name.pyc')
+        self.run_bzr('clean-tree --unknown --ignored --force')
+        self.failIfExists('name')
+        self.failIfExists('name~')
+        self.failIfExists('name.pyc')

=== added file 'bzrlib/tests/test_clean_tree.py'
--- bzrlib/tests/test_clean_tree.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_clean_tree.py	2009-03-10 00:33:54 +0000
@@ -0,0 +1,78 @@
+# Copyright (C) 2005, 2009 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import os
+from StringIO import StringIO
+
+from bzrlib.bzrdir import (
+    BzrDir,
+    )
+from bzrlib.clean_tree import (
+    clean_tree,
+    iter_deletables,
+    )
+from bzrlib.osutils import (
+    has_symlinks,
+    )
+from bzrlib.tests import (
+    TestCaseInTempDir,
+    )
+
+
+class TestCleanTree(TestCaseInTempDir):
+
+    def test_symlinks(self):
+        if has_symlinks() is False:
+            return
+        os.mkdir('branch')
+        BzrDir.create_standalone_workingtree('branch')
+        os.symlink(os.path.realpath('no-die-please'), 'branch/die-please')
+        os.mkdir('no-die-please')
+        self.failUnlessExists('branch/die-please')
+        os.mkdir('no-die-please/child')
+
+        clean_tree('branch', unknown=True, no_prompt=True)
+        self.failUnlessExists('no-die-please')
+        self.failUnlessExists('no-die-please/child')
+
+    def test_iter_deletable(self):
+        """Files are selected for deletion appropriately"""
+        os.mkdir('branch')
+        tree = BzrDir.create_standalone_workingtree('branch')
+        transport = tree.bzrdir.root_transport
+        transport.put_bytes('.bzrignore', '*~\n*.pyc\n.bzrignore\n')
+        transport.put_bytes('file.BASE', 'contents')
+        tree.lock_write()
+        try:
+            self.assertEqual(len(list(iter_deletables(tree, unknown=True))), 1)
+            transport.put_bytes('file', 'contents')
+            transport.put_bytes('file~', 'contents')
+            transport.put_bytes('file.pyc', 'contents')
+            dels = sorted([r for a,r in iter_deletables(tree, unknown=True)])
+            self.assertEqual(['file', 'file.BASE'], dels)
+
+            dels = [r for a,r in iter_deletables(tree, detritus=True)]
+            self.assertEqual(sorted(['file~', 'file.BASE']), dels)
+
+            dels = [r for a,r in iter_deletables(tree, ignored=True)]
+            self.assertEqual(sorted(['file~', 'file.pyc', '.bzrignore']),
+                             dels)
+
+            dels = [r for a,r in iter_deletables(tree, unknown=False)]
+            self.assertEqual([], dels)
+        finally:
+            tree.unlock()

